@@ -44,13 +44,14 @@ log_message(const char *fmt, ...)
 	static int logfd = -1;
 	char time_str[20];
 	time_t now;
+	struct tm tm;
 	va_list args;
 
 	if (!log_file)
 		return;
 
 	if (logfd < 0) {
-		logfd = open(log_file, O_CREAT | O_APPEND, 0644);
+		logfd = open(log_file, O_WRONLY | O_CREAT | O_APPEND, 0644);
 		if (logfd < 0) {
 			notice(_("could not open log '%s': %s"),
 			       log_file, strerror(errno));
@@ -61,19 +62,23 @@ log_message(const char *fmt, ...)
 	}
 
 	time(&now);
-	strftime(time_str, sizeof(time_str), "%Y-%m-%d %H:%M:%S",
-	         localtime(&now));
+	if (localtime_r(&now, &tm) == NULL) {
+		notice(_("cannot get local time to log into '%s': %s"),
+		       log_file, strerror(errno));
+		return;
+	}
+	strftime(time_str, sizeof(time_str), "%Y-%m-%d %H:%M:%S", &tm);
 
 	va_start(args, fmt);
-	varbuf_reset(&log);
-	varbuf_add_str(&log, time_str);
+	varbuf_set_str(&log, time_str);
 	varbuf_add_char(&log, ' ');
-	varbuf_vprintf(&log, fmt, args);
+	varbuf_add_vfmt(&log, fmt, args);
 	varbuf_add_char(&log, '\n');
-	varbuf_end_str(&log);
 	va_end(args);
 
-	fd_write(logfd, log.buf, log.size);
+	if (fd_write(logfd, log.buf, log.used) < 0)
+		notice(_("cannot write to log file '%s': %s"),
+		       log_file, strerror(errno));
 }
 
 struct pipef {
@@ -107,8 +112,7 @@ statusfd_send(const char *fmt, ...)
 		return;
 
 	va_start(args, fmt);
-	varbuf_reset(&vb);
-	varbuf_vprintf(&vb, fmt, args);
+	varbuf_set_vfmt(&vb, fmt, args);
 	/* Sanitize string to not include new lines, as front-ends should be
 	 * doing their own word-wrapping. */
 	varbuf_map_char(&vb, '\n', ' ');
