@@ -44,13 +44,18 @@ typedef struct pkginfo *pkg_mapper_func(const char *name);
 typedef void pkg_array_visitor_func(struct pkg_array *a, struct pkginfo *pkg,
                                     void *pkg_data);
 
-void pkg_array_init_from_hash(struct pkg_array *a);
-void pkg_array_init_from_names(struct pkg_array *a, pkg_mapper_func *pkg_mapper,
-                               const char **pkg_names);
-void pkg_array_foreach(struct pkg_array *a, pkg_array_visitor_func *pkg_visitor,
-                       void *pkg_data);
-void pkg_array_sort(struct pkg_array *a, pkg_sorter_func *pkg_sort);
-void pkg_array_destroy(struct pkg_array *a);
+void
+pkg_array_init_from_hash(struct pkg_array *a);
+void
+pkg_array_init_from_names(struct pkg_array *a, pkg_mapper_func *pkg_mapper,
+                          const char **pkg_names);
+void
+pkg_array_foreach(struct pkg_array *a, pkg_array_visitor_func *pkg_visitor,
+                  void *pkg_data);
+void
+pkg_array_sort(struct pkg_array *a, pkg_sorter_func *pkg_sort);
+void
+pkg_array_destroy(struct pkg_array *a);
 
 /** @} */
 
