@@ -1,4 +1,5 @@
 # Copyright © 2008-2009 Raphaël Hertzog <hertzog@debian.org>
+# Copyright © 2008-2009, 2012-2017, 2022 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,33 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-package Dpkg::Vendor;
-
-use strict;
-use warnings;
-use feature qw(state);
-
-our $VERSION = '1.01';
-our @EXPORT_OK = qw(
-    get_current_vendor
-    get_vendor_info
-    get_vendor_file
-    get_vendor_dir
-    get_vendor_object
-    run_vendor_hook
-);
-
-use Exporter qw(import);
-
-use Dpkg ();
-use Dpkg::ErrorHandling;
-use Dpkg::Gettext;
-use Dpkg::Build::Env;
-use Dpkg::Control::HashCore;
-
-my $origins = "$Dpkg::CONFDIR/origins";
-$origins = $ENV{DPKG_ORIGINS_DIR} if $ENV{DPKG_ORIGINS_DIR};
-
 =encoding utf8
 
 =head1 NAME
@@ -66,11 +40,38 @@ is to name the vendor file using the vendor name in all lowercase, but some
 variation is permitted. Namely, spaces are mapped to dashes ('-'), and the
 file can have the same casing as the Vendor field, or it can be capitalized.
 
+=cut
+
+package Dpkg::Vendor 1.03;
+
+use v5.36;
+
+our @EXPORT_OK = qw(
+    get_current_vendor
+    get_vendor_info
+    get_vendor_file
+    get_vendor_dir
+    get_vendor_object
+    run_vendor_hook
+);
+
+use Exporter qw(import);
+use List::Util qw(uniq);
+
+use Dpkg ();
+use Dpkg::ErrorHandling;
+use Dpkg::Gettext;
+use Dpkg::BuildEnv;
+use Dpkg::Control::HashCore;
+
+my $origins = "$Dpkg::CONFDIR/origins";
+$origins = $ENV{DPKG_ORIGINS_DIR} if $ENV{DPKG_ORIGINS_DIR};
+
 =head1 FUNCTIONS
 
 =over 4
 
-=item $dir = Dpkg::Vendor::get_vendor_dir()
+=item $dir = get_vendor_dir()
 
 Returns the current dpkg origins directory name, where the vendor files
 are stored.
@@ -81,9 +82,9 @@ sub get_vendor_dir {
     return $origins;
 }
 
-=item $fields = Dpkg::Vendor::get_vendor_info($name)
+=item $fields = get_vendor_info([$name])
 
-Returns a Dpkg::Control object with the information parsed from the
+Returns a L<Dpkg::Control> object with the information parsed from the
 corresponding vendor file in $Dpkg::CONFDIR/origins/. If $name is omitted,
 it will use $Dpkg::CONFDIR/origins/default which is supposed to be a symlink
 to the vendor of the currently installed operating system. Returns undef
@@ -91,40 +92,49 @@ if there's no file for the given vendor.
 
 =cut
 
-sub get_vendor_info(;$) {
+my $vendor_sep_regex = qr{[^A-Za-z0-9]+};
+
+sub get_vendor_info {
     my $vendor = shift || 'default';
+    my $vendor_key = lc $vendor =~ s{$vendor_sep_regex}{}gr;
     state %VENDOR_CACHE;
-    return $VENDOR_CACHE{$vendor} if exists $VENDOR_CACHE{$vendor};
+    return $VENDOR_CACHE{$vendor_key} if exists $VENDOR_CACHE{$vendor_key};
 
     my $file = get_vendor_file($vendor);
     return unless $file;
     my $fields = Dpkg::Control::HashCore->new();
-    $fields->load($file) or error(g_('%s is empty'), $file);
-    $VENDOR_CACHE{$vendor} = $fields;
+    $fields->load($file, compression => 0) or error(g_('%s is empty'), $file);
+    $VENDOR_CACHE{$vendor_key} = $fields;
     return $fields;
 }
 
-=item $name = Dpkg::Vendor::get_vendor_file($name)
+=item $name = get_vendor_file([$name])
 
 Check if there's a file for the given vendor and returns its
 name.
 
+The vendor filename will be derived from the vendor name, by replacing any
+number of non-alphanumeric characters (that is B<[^A-Za-z0-9]>) into "B<->",
+then the resulting name will be tried in sequence by lower-casing it,
+keeping it as is, lower-casing then capitalizing it, and capitalizing it.
+
 =cut
 
-sub get_vendor_file(;$) {
+sub get_vendor_file {
     my $vendor = shift || 'default';
-    my $file;
-    my @tries = ($vendor, lc($vendor), ucfirst($vendor), ucfirst(lc($vendor)));
-    if ($vendor =~ s/\s+/-/) {
-        push @tries, $vendor, lc($vendor), ucfirst($vendor), ucfirst(lc($vendor));
+
+    my @names;
+    my $vendor_sep = $vendor =~ s{$vendor_sep_regex}{-}gr;
+    push @names, lc $vendor_sep, $vendor_sep, ucfirst lc $vendor_sep, ucfirst $vendor_sep;
+
+    foreach my $name (uniq @names) {
+        next unless -e "$origins/$name";
+        return "$origins/$name";
     }
-    foreach my $name (@tries) {
-        $file = "$origins/$name" if -e "$origins/$name";
-    }
-    return $file;
+    return;
 }
 
-=item $name = Dpkg::Vendor::get_current_vendor()
+=item $name = get_current_vendor()
 
 Returns the name of the current vendor. If DEB_VENDOR is set, it uses
 that first, otherwise it falls back to parsing $Dpkg::CONFDIR/origins/default.
@@ -132,10 +142,10 @@ If that file doesn't exist, it returns undef.
 
 =cut
 
-sub get_current_vendor() {
+sub get_current_vendor {
     my $f;
-    if (Dpkg::Build::Env::has('DEB_VENDOR')) {
-        $f = get_vendor_info(Dpkg::Build::Env::get('DEB_VENDOR'));
+    if (Dpkg::BuildEnv::has('DEB_VENDOR')) {
+        $f = get_vendor_info(Dpkg::BuildEnv::get('DEB_VENDOR'));
         return $f->{'Vendor'} if defined $f;
     }
     $f = get_vendor_info();
@@ -143,33 +153,43 @@ sub get_current_vendor() {
     return;
 }
 
-=item $object = Dpkg::Vendor::get_vendor_object($name)
+=item $object = get_vendor_object($name)
 
 Return the Dpkg::Vendor::* object of the corresponding vendor.
 If $name is omitted, return the object of the current vendor.
-If no vendor can be identified, then return the Dpkg::Vendor::Default
+If no vendor can be identified, then return the L<Dpkg::Vendor::Default>
 object.
 
+The module name will be derived from the vendor name, by splitting parts
+around groups of non alphanumeric character (that is B<[^A-Za-z0-9]>)
+separators, by either capitalizing or lower-casing and capitalizing each part
+and then joining them without the separators. So the expected casing is based
+on the one from the B<Vendor> field in the F<origins> file.
+
 =cut
 
 sub get_vendor_object {
     my $vendor = shift || get_current_vendor() || 'Default';
+    my $vendor_key = lc $vendor =~ s{$vendor_sep_regex}{}gr;
     state %OBJECT_CACHE;
-    return $OBJECT_CACHE{$vendor} if exists $OBJECT_CACHE{$vendor};
+    return $OBJECT_CACHE{$vendor_key} if exists $OBJECT_CACHE{$vendor_key};
 
-    my ($obj, @names);
-    push @names, $vendor, lc($vendor), ucfirst($vendor), ucfirst(lc($vendor));
+    my @vendor_parts = split m{$vendor_sep_regex}, $vendor;
 
-    foreach my $name (@names) {
+    my @names;
+    push @names, join q{}, map { ucfirst } @vendor_parts;
+    push @names, join q{}, map { ucfirst lc } @vendor_parts;
+
+    foreach my $name (uniq @names) {
+        my $module = "Dpkg::Vendor::$name";
         eval qq{
-            pop \@INC if \$INC[-1] eq '.';
-            require Dpkg::Vendor::$name;
-            \$obj = Dpkg::Vendor::$name->new();
+            require $module;
         };
-        unless ($@) {
-            $OBJECT_CACHE{$vendor} = $obj;
-            return $obj;
-        }
+        next if $@;
+
+        my $obj = $module->new();
+        $OBJECT_CACHE{$vendor_key} = $obj;
+        return $obj;
     }
 
     my $info = get_vendor_info($vendor);
@@ -180,21 +200,34 @@ sub get_vendor_object {
     }
 }
 
-=item Dpkg::Vendor::run_vendor_hook($hookid, @params)
+=item run_vendor_hook($hookid, @params)
 
 Run a hook implemented by the current vendor object.
 
 =cut
 
 sub run_vendor_hook {
+    my @args = @_;
     my $vendor_obj = get_vendor_object();
-    $vendor_obj->run_hook(@_);
+
+    $vendor_obj->run_hook(@args);
 }
 
 =back
 
 =head1 CHANGES
 
+=head2 Version 1.03 (dpkg 1.22.12)
+
+Obsolete behavior: get_vendor_file() and get_vendor_object() no longer
+support the deprecated behavior from 1.02.
+
+=head2 Version 1.02 (dpkg 1.21.10)
+
+Deprecated behavior: get_vendor_file() loading vendor files with no special
+characters remapping. get_vendor_object() loading vendor module names with
+no special character stripping.
+
 =head2 Version 1.01 (dpkg 1.17.0)
 
 New function: get_vendor_dir().
@@ -205,7 +238,7 @@ Mark the module as public.
 
 =head1 SEE ALSO
 
-deb-origin(5).
+L<deb-origin(5)>.
 
 =cut
 
